package org.eclipse.equinox.launcher;

import com.jiuqi.dna.launcher.Constants;
import com.jiuqi.dna.launcher.ServerDaemon;
import com.jiuqi.util.ArrayUtils;
import com.jiuqi.util.BundleUtils;
import com.jiuqi.util.StringUtil;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.net.URLStreamHandlerFactory;
import java.security.AllPermission;
import java.security.CodeSource;
import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Policy;
import java.security.ProtectionDomain;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class Main implements Constants {
   protected boolean debug = false;
   protected String bootLocation = null;
   protected URL installLocation = null;
   protected URL configurationLocation = null;
   protected String parentConfigurationLocation = null;
   protected String framework = "org.eclipse.osgi";
   protected String devClassPath = null;
   private Properties devClassPathProps = null;
   protected boolean inDevelopmentMode = false;
   protected String os = null;
   protected String ws = null;
   protected String arch = null;
   private String library = null;
   private String exitData = null;
   private String vm = null;
   private String[] vmargs = null;
   private String[] commands = null;
   String[] extensionPaths = null;
   JNIBridge bridge = null;
   private boolean showSplash = false;
   private String splashLocation = null;
   private String endSplash = null;
   private boolean initialize = false;
   private boolean splashDown = false;
   private final Thread splashHandler = new SplashHandler();
   public static final String SPLASH_HANDLE = "org.eclipse.equinox.launcher.splash.handle";
   public static final String SPLASH_LOCATION = "org.eclipse.equinox.launcher.splash.location";
   private static final String FRAMEWORK = "-framework";
   private static final String INSTALL = "-install";
   private static final String INITIALIZE = "-initialize";
   private static final String VM = "-vm";
   private static final String VMARGS = "-vmargs";
   private static final String DEBUG = "-debug";
   private static final String DEV = "-dev";
   private static final String CONFIGURATION = "-configuration";
   private static final String NOSPLASH = "-nosplash";
   private static final String SHOWSPLASH = "-showsplash";
   private static final String EXITDATA = "-exitdata";
   private static final String NAME = "-name";
   private static final String LAUNCHER = "-launcher";
   private static final String LIBRARY = "--launcher.library";
   private static final String NL = "-nl";
   private static final String ENDSPLASH = "-endsplash";
   private static final String SPLASH_IMAGE = "splash.bmp";
   private static final String CLEAN = "-clean";
   private static final String NOEXIT = "-noExit";
   private static final String OS = "-os";
   private static final String WS = "-ws";
   private static final String REDIRECT = "-redirect";
   private static final String ARCH = "-arch";
   private static final String STARTUP = "-startup";
   private static final String OSGI = "org.eclipse.osgi";
   private static final String STARTER = "org.eclipse.core.runtime.adaptor.EclipseStarter";
   private static final String PLATFORM_URL = "platform:/base/";
   private static final String ECLIPSE_PROPERTIES = "eclipse.properties";
   private static final String FILE_SCHEME = "file:";
   protected static final String REFERENCE_SCHEME = "reference:";
   protected static final String JAR_SCHEME = "jar:";
   private static final String CONFIG_DIR = "configuration/";
   private static final String CONFIG_FILE = "config.ini";
   private static final String CONFIG_FILE_TEMP_SUFFIX = ".tmp";
   private static final String CONFIG_FILE_BAK_SUFFIX = ".bak";
   private static final String ECLIPSE = "eclipse";
   private static final String PRODUCT_SITE_MARKER = ".eclipseproduct";
   private static final String PRODUCT_SITE_ID = "id";
   private static final String PRODUCT_SITE_VERSION = "version";
   private static final String PROP_USER_HOME = "user.home";
   private static final String PROP_USER_DIR = "user.dir";
   private static final String PROP_INSTALL_AREA = "osgi.install.area";
   private static final String PROP_OS_TYPE = "os.type";
   private static final String PROP_REDIRECT_OUT = "out.redirect";
   private static final String PROP_CONSOLE_BLOCK = "osgi.console.blockOnReady";
   private static final String PROP_CONFIG_AREA = "osgi.configuration.area";
   private static final String PROP_CONFIG_AREA_DEFAULT = "osgi.configuration.area.default";
   private static final String PROP_BASE_CONFIG_AREA = "osgi.baseConfiguration.area";
   private static final String PROP_SHARED_CONFIG_AREA = "osgi.sharedConfiguration.area";
   private static final String PROP_CONFIG_CASCADED = "osgi.configuration.cascaded";
   protected static final String PROP_FRAMEWORK = "osgi.framework";
   private static final String PROP_SPLASHPATH = "osgi.splashPath";
   private static final String PROP_SPLASHLOCATION = "osgi.splashLocation";
   private static final String PROP_CLASSPATH = "osgi.frameworkClassPath";
   private static final String PROP_EXTENSIONS = "osgi.framework.extensions";
   private static final String PROP_FRAMEWORK_SYSPATH = "osgi.syspath";
   private static final String PROP_FRAMEWORK_SHAPE = "osgi.framework.shape";
   private static final String PROP_LOGFILE = "osgi.logfile";
   private static final String PROP_REQUIRED_JAVA_VERSION = "osgi.requiredJavaVersion";
   private static final String PROP_PARENT_CLASSLOADER = "osgi.parentClassloader";
   private static final String PROP_FRAMEWORK_PARENT_CLASSLOADER = "osgi.frameworkParentClassloader";
   private static final String PROP_NL = "osgi.nl";
   static final String PROP_NOSHUTDOWN = "osgi.noShutdown";
   private static final String PROP_DEBUG = "osgi.debug";
   private static final String PROP_EXITCODE = "eclipse.exitcode";
   private static final String PROP_EXITDATA = "eclipse.exitdata";
   private static final String PROP_VM = "eclipse.vm";
   private static final String PROP_VMARGS = "eclipse.vmargs";
   private static final String PROP_COMMANDS = "eclipse.commands";
   private static final String PROP_ECLIPSESECURITY = "eclipse.security";
   private static final String NONE = "@none";
   private static final String NO_DEFAULT = "@noDefault";
   private static final String USER_HOME = "@user.home";
   private static final String USER_DIR = "@user.dir";
   private static final String PARENT_CLASSLOADER_APP = "app";
   private static final String PARENT_CLASSLOADER_EXT = "ext";
   private static final String PARENT_CLASSLOADER_BOOT = "boot";
   private static final String PARENT_CLASSLOADER_CURRENT = "current";
   protected static final String SESSION = "!SESSION";
   protected static final String ENTRY = "!ENTRY";
   protected static final String MESSAGE = "!MESSAGE";
   protected static final String STACK = "!STACK";
   protected static final int ERROR = 4;
   protected static final String PLUGIN_ID = "com.jiuqi.dna.launcher";
   protected File logFile = null;
   protected BufferedWriter log = null;
   protected boolean newSession = true;
   private static final String[] bundleDirs = new String[]{"app", "dna", "thr", "lib", "bundles"};
   private static final String prefix2 = "reference:file:..";
   private static final String SUBFIX_START = "@start";
   private static final String[] notLoadBundles = new String[]{"org.eclipse.osgi", "com.jiuqi.dna.launcher"};

   private String getWS() {
      if (this.ws != null) {
         return this.ws;
      } else {
         String os = this.getOS();
         if (os.equals("win32")) {
            return "win32";
         } else if (os.equals("linux")) {
            return "gtk";
         } else if (os.equals("macosx")) {
            return "carbon";
         } else if (os.equals("hpux")) {
            return "motif";
         } else if (os.equals("aix")) {
            return "motif";
         } else if (os.equals("solaris")) {
            return "motif";
         } else {
            return os.equals("qnx") ? "photon" : "unknown";
         }
      }
   }

   private String getOS() {
      if (this.os != null) {
         return this.os;
      } else {
         String osName = System.getProperties().getProperty("os.name");
         if (osName.regionMatches(true, 0, "win32", 0, 3)) {
            return "win32";
         } else if (osName.equalsIgnoreCase("SunOS")) {
            return "solaris";
         } else if (osName.equalsIgnoreCase("Linux")) {
            return "linux";
         } else if (osName.equalsIgnoreCase("QNX")) {
            return "qnx";
         } else if (osName.equalsIgnoreCase("AIX")) {
            return "aix";
         } else if (osName.equalsIgnoreCase("HP-UX")) {
            return "hpux";
         } else {
            return osName.regionMatches(true, 0, "Mac OS", 0, "Mac OS".length()) ? "macosx" : "unknown";
         }
      }
   }

   private String getArch() {
      if (this.arch != null) {
         return this.arch;
      } else {
         String name = System.getProperties().getProperty("os.arch");
         if (name.equalsIgnoreCase("i386")) {
            return "x86";
         } else {
            return name.equalsIgnoreCase("amd64") ? "x86_64" : name;
         }
      }
   }

   private void setupJNI(URL[] defaultPath) {
      String libPath = null;
      if (this.library != null) {
         File lib = new File(this.library);
         if (lib.isDirectory()) {
            libPath = this.searchFor("eclipse", lib.getAbsolutePath());
         } else if (lib.exists()) {
            libPath = lib.getAbsolutePath();
         }
      }

      if (libPath == null) {
         String fragmentOS = this.getOS();
         StringBuffer buffer = new StringBuffer("com.jiuqi.dna.launcher");
         buffer.append('.');
         buffer.append(this.getWS());
         buffer.append('.');
         buffer.append(fragmentOS);
         if (!fragmentOS.equals("macosx")) {
            buffer.append('.');
            buffer.append(this.getArch());
         }

         String fragmentName = buffer.toString();
         String fragment = null;
         File entryFile;
         String dir;
         if (this.inDevelopmentMode) {
            String devPathList = this.devClassPathProps.getProperty("com.jiuqi.dna.launcher");
            String[] locations = this.getArrayFromList(devPathList);
            if (locations.length > 0) {
               entryFile = new File(locations[0]);
               if (entryFile.isAbsolute()) {
                  dir = entryFile.getParent();
                  fragment = this.searchFor(fragmentName, dir);
                  if (fragment != null) {
                     libPath = this.searchFor("eclipse", fragment);
                  }
               }
            }
         }

         if (libPath == null && this.bootLocation != null) {
            URL[] urls = defaultPath;
            if (defaultPath != null && defaultPath.length > 0) {
               for(int i = defaultPath.length - 1; i >= 0 && libPath == null; --i) {
                  entryFile = new File(urls[i].getFile());
                  dir = entryFile.getParent();
                  if (this.inDevelopmentMode) {
                     String devDir = dir + "/" + "com.jiuqi.dna.launcher" + "/fragments";
                     fragment = this.searchFor(fragmentName, devDir);
                  }

                  if (fragment == null) {
                     fragment = this.searchFor(fragmentName, dir);
                  }

                  if (fragment != null) {
                     libPath = this.searchFor("eclipse", fragment);
                  }
               }
            }
         }

         if (libPath == null) {
            URL install = this.getInstallLocation();
            String location = install.getFile();
            location = location + "/bundles/";
            if ((new File(location)).exists()) {
               fragment = this.searchFor(fragmentName, location);
            } else {
               location = location + "/thr/";
               fragment = this.searchFor(fragmentName, location);
            }

            if (fragment != null) {
               libPath = this.searchFor("eclipse", fragment);
            }
         }
      }

      this.library = libPath;
      if (this.library != null) {
         this.bridge = new JNIBridge(this.library);
      }

   }

   protected void basicRun(String[] args) throws Exception {
      System.getProperties().put("eclipse.startTime", Long.toString(System.currentTimeMillis()));
      String[] args2 = new String[]{"-os", System.getProperty("os.name"), "-console"};
      this.commands = args2;
      String[] passThruArgs = this.processCommandLine(args2);
      if (!this.debug) {
         this.debug = System.getProperty("osgi.debug") != null;
      }

      this.setupVMProperties();
      this.getInstallLocation();
      String installPath = System.getProperty("osgi.install.area");
      if (installPath != null) {
         URL url = new URL(installPath);
         File f = new File(url.getFile());
         installPath = f.getAbsolutePath();
         System.setProperty("com.jiuqi.dna.rootpath", installPath);
         System.getProperties().put("osgi.configuration.area", installPath + "/work");
      }

      this.processConfiguration2();
      if (args.length > 0 && "stop".equals(args[0])) {
         ServerDaemon.stopServer();
      } else if (args.length > 0 && "restart".equals(args[0])) {
         ServerDaemon.restartServer();
      } else {
         ServerDaemon.startServer();
      }

      if (Boolean.getBoolean("out.redirect")) {
         IORedirecter.redirectIO(installPath);
      }

      if (Boolean.getBoolean("osgi.console.blockOnReady")) {
         System.setProperty("osgi.console.blockOnReady", "");
      }

      URL[] bootPath = this.getBootPath(this.bootLocation);
      this.setupJNI(bootPath);
      if (this.checkVersion(System.getProperty("java.version"), System.getProperty("osgi.requiredJavaVersion"))) {
         this.setSecurityPolicy(bootPath);
         this.handleSplash(bootPath);
         this.beforeFwkInvocation();
         this.invokeFramework(passThruArgs, bootPath);
      }
   }

   protected void beforeFwkInvocation() {
   }

   private final void processConfiguration2() {
      System.setProperty("osgi.clean", "true");
      System.setProperty("org.eclipse.equinox.http.jetty.autostart", "false");
      System.setProperty("POOL_MAX", "100000");
      System.setProperty("osgi.noShutdown", "true");
      StringBuffer initBundles = new StringBuffer();
      String serverRoot = System.getProperty("com.jiuqi.dna.rootpath");
      File f = new File(serverRoot);
      String serverRootPath = StringUtil.formatPath(f.getPath(), false);
      if (serverRootPath.endsWith("/")) {
         serverRootPath = serverRootPath.substring(0, serverRootPath.length() - 1);
      }

      String[] var8;
      int var7 = (var8 = bundleDirs).length;

      for(int var6 = 0; var6 < var7; ++var6) {
         String bundleDir = var8[var6];
         File fBundleDir = new File(serverRoot, bundleDir);
         this.doStruct(fBundleDir, initBundles, serverRootPath);
      }

      if (Boolean.getBoolean("launcher.debug")) {
         System.out.println(initBundles.toString());
      }

      System.setProperty("osgi.bundles", initBundles.toString());
   }

   private final void doStruct(File fBundleDir, StringBuffer configBuffer, String serverRootPath) {
      if (BundleUtils.isBundle(fBundleDir)) {
         String bundleId = BundleUtils.getBundleId(fBundleDir);
         if (this.isNotLoadBundles(bundleId)) {
            return;
         }

         String filePath = StringUtil.formatPath(fBundleDir.getPath(), false);
         filePath = filePath.substring(serverRootPath.length());
         if (this.canStartBundle(fBundleDir)) {
            filePath = "reference:file:.." + filePath + "@start";
         } else {
            filePath = "reference:file:.." + filePath;
         }

         configBuffer.append(filePath);
         configBuffer.append(",");
      } else if (fBundleDir.isDirectory()) {
         File[] var7;
         int var6 = (var7 = fBundleDir.listFiles()).length;

         for(int var9 = 0; var9 < var6; ++var9) {
            File f = var7[var9];
            this.doStruct(f, configBuffer, serverRootPath);
         }
      }

   }

   private final boolean isNotLoadBundles(String bundleId) {
      return ArrayUtils.contains(notLoadBundles, bundleId);
   }

   private boolean canStartBundle(File bundleFile) {
      return !BundleUtils.isFragmentBundle(bundleFile);
   }

   protected void setSecurityPolicy(URL[] bootPath) {
      String eclipseSecurity = System.getProperty("eclipse.security");
      if (eclipseSecurity != null) {
         SecurityManager sm = System.getSecurityManager();
         boolean setSM = false;
         if (sm == null) {
            if (eclipseSecurity.length() < 1) {
               eclipseSecurity = "java.lang.SecurityManager";
            }

            try {
               Class clazz = Class.forName(eclipseSecurity);
               sm = (SecurityManager)clazz.newInstance();
               setSM = true;
            } catch (Throwable var9) {
               System.getProperties().put("java.security.manager", eclipseSecurity);
            }
         }

         ProtectionDomain domain = Main.class.getProtectionDomain();
         CodeSource source = null;
         if (domain != null) {
            source = Main.class.getProtectionDomain().getCodeSource();
         }

         if (domain == null || source == null) {
            this.log("Can not automatically set the security manager. Please use a policy file.");
            return;
         }

         URL[] rootURLs = new URL[bootPath.length + 1];
         rootURLs[0] = source.getLocation();
         System.arraycopy(bootPath, 0, rootURLs, 1, bootPath.length);
         Policy eclipsePolicy = new EclipsePolicy(Policy.getPolicy(), rootURLs);
         Policy.setPolicy(eclipsePolicy);
         if (setSM) {
            System.setSecurityManager(sm);
         }
      }

   }

   private void invokeFramework(String[] passThruArgs, URL[] bootPath) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, Error, Exception, InvocationTargetException {
      String type = System.getProperty("osgi.frameworkParentClassloader", System.getProperty("osgi.parentClassloader", "boot"));
      ClassLoader parent = null;
      if ("app".equalsIgnoreCase(type)) {
         parent = ClassLoader.getSystemClassLoader();
      } else if ("ext".equalsIgnoreCase(type)) {
         ClassLoader appCL = ClassLoader.getSystemClassLoader();
         if (appCL != null) {
            parent = appCL.getParent();
         }
      } else if ("current".equalsIgnoreCase(type)) {
         parent = this.getClass().getClassLoader();
      }

      URLClassLoader loader = new StartupClassLoader(bootPath, parent);
      Class clazz = loader.loadClass("org.eclipse.core.runtime.adaptor.EclipseStarter");
      Method method = clazz.getDeclaredMethod("run", String[].class, Runnable.class);

      try {
         method.invoke(clazz, passThruArgs, this.splashHandler);
      } catch (InvocationTargetException var9) {
         InvocationTargetException e = var9;
         if (e.getTargetException() instanceof Error) {
            throw (Error)e.getTargetException();
         } else if (e.getTargetException() instanceof Exception) {
            throw (Exception)e.getTargetException();
         } else {
            throw e;
         }
      }
   }

   private boolean checkVersion(String availableVersion, String requiredVersion) {
      if (requiredVersion != null && availableVersion != null) {
         try {
            Identifier required = new Identifier(requiredVersion);
            Identifier available = new Identifier(availableVersion);
            boolean compatible = available.isGreaterEqualTo(required);
            if (!compatible) {
               System.getProperties().put("eclipse.exitcode", "14");
               System.getProperties().put("eclipse.exitdata", "<title>Incompatible JVM</title>Version " + availableVersion + " of the JVM is not suitable for this product. Version: " + requiredVersion + " or greater is required.");
            }

            return compatible;
         } catch (SecurityException var6) {
            return true;
         } catch (NumberFormatException var7) {
            return true;
         }
      } else {
         return true;
      }
   }

   protected String decode(String urlString) {
      int encodedLength;
      int decodedLength;
      int i;
      try {
         Class clazz = URLDecoder.class;
         Method method = clazz.getDeclaredMethod("decode", String.class, String.class);
         if (urlString.indexOf(43) >= 0) {
            encodedLength = urlString.length();
            StringBuffer buf = new StringBuffer(encodedLength);
            decodedLength = 0;

            while(true) {
               if (decodedLength >= encodedLength) {
                  urlString = buf.toString();
                  break;
               }

               i = urlString.charAt(decodedLength);
               if (i == 43) {
                  buf.append("%2B");
               } else {
                  buf.append((char)i);
               }

               ++decodedLength;
            }
         }

         Object result = method.invoke((Object)null, urlString, "UTF-8");
         if (result != null) {
            return (String)result;
         }
      } catch (Exception var12) {
      }

      boolean replaced = false;
      byte[] encodedBytes = urlString.getBytes();
      encodedLength = encodedBytes.length;
      byte[] decodedBytes = new byte[encodedLength];
      decodedLength = 0;

      for(i = 0; i < encodedLength; ++i) {
         byte b = encodedBytes[i];
         if (b == 37) {
            if (i + 2 >= encodedLength) {
               throw new IllegalArgumentException("Malformed URL (\"" + urlString + "\"): % must be followed by 2 digits.");
            }

            ++i;
            byte enc1 = encodedBytes[i];
            ++i;
            byte enc2 = encodedBytes[i];
            b = (byte)((this.hexToByte(enc1) << 4) + this.hexToByte(enc2));
            replaced = true;
         }

         decodedBytes[decodedLength++] = b;
      }

      if (!replaced) {
         return urlString;
      } else {
         try {
            return new String(decodedBytes, 0, decodedLength, "UTF-8");
         } catch (UnsupportedEncodingException var11) {
            return new String(decodedBytes, 0, decodedLength);
         }
      }
   }

   protected String[] getArrayFromList(String prop) {
      if (prop != null && !prop.trim().equals("")) {
         Vector list = new Vector();
         StringTokenizer tokens = new StringTokenizer(prop, ",");

         while(tokens.hasMoreTokens()) {
            String token = tokens.nextToken().trim();
            if (!token.equals("")) {
               list.addElement(token);
            }
         }

         return list.isEmpty() ? new String[0] : (String[])list.toArray(new String[list.size()]);
      } else {
         return new String[0];
      }
   }

   private URL[] getDevPath(URL base) throws IOException {
      ArrayList result = new ArrayList(5);
      if (this.inDevelopmentMode) {
         this.addDevEntries(base, result, "org.eclipse.osgi");
      }

      this.addBaseJars(base, result);
      return (URL[])result.toArray(new URL[result.size()]);
   }

   URL constructURL(URL url, String name) {
      String externalForm = url.toExternalForm();
      if (externalForm.endsWith(".jar")) {
         try {
            return new URL("jar:" + url + "!/" + name);
         } catch (MalformedURLException var5) {
         }
      }

      try {
         return new URL(url, name);
      } catch (MalformedURLException var4) {
         return null;
      }
   }

   private void readFrameworkExtensions(URL base, ArrayList result) throws IOException {
      String[] extensions = this.getArrayFromList(System.getProperties().getProperty("osgi.framework.extensions"));
      String parent = (new File(base.getFile())).getParent().toString();
      ArrayList extensionResults = new ArrayList(extensions.length);

      for(int i = 0; i < extensions.length; ++i) {
         String path = this.searchFor(extensions[i], parent);
         if (path == null) {
            this.log("Could not find extension: " + extensions[i]);
         } else {
            if (this.debug) {
               System.out.println("Loading extension: " + extensions[i]);
            }

            URL extensionURL = null;
            if (this.installLocation.getProtocol().equals("file")) {
               extensionResults.add(path);
               extensionURL = (new File(path)).toURL();
            } else {
               extensionURL = new URL(this.installLocation.getProtocol(), this.installLocation.getHost(), this.installLocation.getPort(), path);
            }

            Properties extensionProperties = null;

            try {
               extensionProperties = this.loadProperties(this.constructURL(extensionURL, "eclipse.properties"));
            } catch (IOException var14) {
               if (this.debug) {
                  System.out.println("\teclipse.properties not found");
               }
            }

            String extensionClassPath = null;
            if (extensionProperties != null) {
               extensionClassPath = extensionProperties.getProperty("osgi.frameworkClassPath");
            } else {
               extensionProperties = new Properties();
            }

            String[] entries = extensionClassPath != null && extensionClassPath.length() != 0 ? this.getArrayFromList(extensionClassPath) : new String[]{""};
            String qualifiedPath;
            if (System.getProperty("osgi.frameworkClassPath") == null) {
               qualifiedPath = ".";
            } else {
               qualifiedPath = "";
            }

            for(int j = 0; j < entries.length; ++j) {
               qualifiedPath = qualifiedPath + ", file:" + path + entries[j];
            }

            extensionProperties.put("osgi.frameworkClassPath", qualifiedPath);
            this.mergeProperties(System.getProperties(), extensionProperties);
            if (this.inDevelopmentMode) {
               this.addDevEntries(extensionURL, result, extensions[i]);
            }
         }
      }

      this.extensionPaths = (String[])extensionResults.toArray(new String[extensionResults.size()]);
   }

   private void addBaseJars(URL base, ArrayList result) throws IOException {
      String baseJarList = System.getProperty("osgi.frameworkClassPath");
      if (baseJarList == null) {
         this.readFrameworkExtensions(base, result);
         baseJarList = System.getProperties().getProperty("osgi.frameworkClassPath");
      }

      File fwkFile = new File(base.getFile());
      boolean fwkIsDirectory = fwkFile.isDirectory();
      if (fwkIsDirectory) {
         System.getProperties().put("osgi.framework.shape", "folder");
      } else {
         System.getProperties().put("osgi.framework.shape", "jar");
      }

      String fwkPath = (new File((new File(base.getFile())).getParent())).getAbsolutePath();
      if (Character.isUpperCase(fwkPath.charAt(0))) {
         char[] chars = fwkPath.toCharArray();
         chars[0] = Character.toLowerCase(chars[0]);
         fwkPath = new String(chars);
      }

      System.getProperties().put("osgi.syspath", fwkPath);
      String[] baseJars = this.getArrayFromList(baseJarList);
      if (baseJars.length == 0) {
         if (!this.inDevelopmentMode && (new File(base.getFile())).isDirectory()) {
            throw new IOException("Unable to initialize osgi.frameworkClassPath");
         } else {
            this.addEntry(base, result);
         }
      } else {
         for(int i = 0; i < baseJars.length; ++i) {
            String string = baseJars[i];

            try {
               if (string.equals(".")) {
                  this.addEntry(base, result);
               }

               URL url = null;
               if (string.startsWith("file:")) {
                  url = (new File(string.substring(5))).toURL();
               } else {
                  url = new URL(string);
               }

               this.addEntry(url, result);
            } catch (MalformedURLException var11) {
               this.addEntry(new URL(base, string), result);
            }
         }

      }
   }

   protected void addEntry(URL url, List result) {
      if ((new File(url.getFile())).exists()) {
         result.add(url);
      }

   }

   private void addDevEntries(URL base, List result, String symbolicName) throws MalformedURLException {
      if (this.devClassPathProps != null) {
         String devPathList = this.devClassPathProps.getProperty(symbolicName);
         if (devPathList == null) {
            devPathList = this.devClassPathProps.getProperty("*");
         }

         String[] locations = this.getArrayFromList(devPathList);

         for(int i = 0; i < locations.length; ++i) {
            String location = locations[i];
            File path = new File(location);
            URL url;
            if (path.isAbsolute()) {
               url = path.toURL();
            } else {
               char lastChar = location.charAt(location.length() - 1);
               if (!location.endsWith(".jar") && lastChar != '/' && lastChar != '\\') {
                  url = new URL(base, location + "/");
               } else {
                  url = new URL(base, location);
               }
            }

            this.addEntry(url, result);
         }

      }
   }

   protected URL[] getBootPath(String base) throws IOException {
      URL url = null;
      if (base != null) {
         url = buildURL(base, true);
      } else {
         url = this.getInstallLocation();
         String path = null;
         File dir = new File(url.getFile(), "bundles");
         if (dir.exists()) {
            path = dir.getAbsolutePath();
         } else {
            dir = new File(url.getFile(), "thr");
            if (dir.exists()) {
               path = dir.getAbsolutePath();
            }
         }

         path = this.searchFor(this.framework, path);
         if (path == null) {
            throw new RuntimeException("Could not find framework");
         }

         if (url.getProtocol().equals("file")) {
            url = (new File(path)).toURL();
         } else {
            url = new URL(url.getProtocol(), url.getHost(), url.getPort(), path);
         }
      }

      if (System.getProperty("osgi.framework") == null) {
         System.getProperties().put("osgi.framework", url.toExternalForm());
      }

      if (this.debug) {
         System.out.println("Framework located:\n    " + url.toExternalForm());
      }

      URL[] result = this.getDevPath(url);
      if (this.debug) {
         System.out.println("Framework classpath:");

         for(int i = 0; i < result.length; ++i) {
            System.out.println("    " + result[i].toExternalForm());
         }
      }

      return result;
   }

   protected String searchFor(String target, String start) {
      return this.searchFor(target, (String)null, start);
   }

   protected String searchFor(String target, String targetSuffix, String start) {
      String[] candidates = (new File(start)).list();
      if (candidates == null) {
         return null;
      } else {
         ArrayList matches = new ArrayList(2);

         for(int i = 0; i < candidates.length; ++i) {
            if (candidates[i].equals(target) || candidates[i].startsWith(target + "_")) {
               matches.add(candidates[i]);
            }
         }

         String[] names = (String[])matches.toArray(new String[matches.size()]);
         int result = this.findMax(names);
         if (result == -1) {
            return null;
         } else {
            File candidate = new File(start, names[result]);
            return candidate.getAbsolutePath().replace(File.separatorChar, '/') + (candidate.isDirectory() ? "/" : "");
         }
      }
   }

   protected int findMax(String[] candidates) {
      int result = -1;
      Object maxVersion = null;

      for(int i = 0; i < candidates.length; ++i) {
         String name = candidates[i];
         String version = "";
         int index = name.indexOf(95);
         if (index != -1) {
            version = name.substring(index + 1);
         }

         Object currentVersion = this.getVersionElements(version);
         if (maxVersion == null) {
            result = i;
            maxVersion = currentVersion;
         } else if (this.compareVersion((Object[])maxVersion, (Object[])currentVersion) < 0) {
            result = i;
            maxVersion = currentVersion;
         }
      }

      return result;
   }

   private int compareVersion(Object[] left, Object[] right) {
      int result = ((Integer)left[0]).compareTo((Integer)right[0]);
      if (result != 0) {
         return result;
      } else {
         result = ((Integer)left[1]).compareTo((Integer)right[1]);
         if (result != 0) {
            return result;
         } else {
            result = ((Integer)left[2]).compareTo((Integer)right[2]);
            return result != 0 ? result : ((String)left[3]).compareTo((String)right[3]);
         }
      }
   }

   private Object[] getVersionElements(String version) {
      if (version.endsWith(".jar")) {
         version = version.substring(0, version.length() - 4);
      }

      Object[] result = new Object[]{new Integer(0), new Integer(0), new Integer(0), ""};
      StringTokenizer t = new StringTokenizer(version, ".");
      int i = 0;

      while(t.hasMoreTokens() && i < 4) {
         String token = t.nextToken();
         if (i < 3) {
            try {
               result[i++] = new Integer(token);
            } catch (Exception var6) {
               break;
            }
         } else {
            result[i++] = token;
         }
      }

      return result;
   }

   private static URL buildURL(String spec, boolean trailingSlash) {
      if (spec == null) {
         return null;
      } else {
         boolean isFile = spec.startsWith("file:");

         File toAdjust;
         try {
            if (isFile) {
               toAdjust = new File(spec.substring(5));
               return toAdjust.isDirectory() ? adjustTrailingSlash(toAdjust.toURL(), trailingSlash) : toAdjust.toURL();
            } else {
               return new URL(spec);
            }
         } catch (MalformedURLException var5) {
            if (isFile) {
               return null;
            } else {
               try {
                  toAdjust = new File(spec);
                  return toAdjust.isDirectory() ? adjustTrailingSlash(toAdjust.toURL(), trailingSlash) : toAdjust.toURL();
               } catch (MalformedURLException var4) {
                  return null;
               }
            }
         }
      }
   }

   private static URL adjustTrailingSlash(URL url, boolean trailingSlash) throws MalformedURLException {
      String file = url.getFile();
      if (trailingSlash == file.endsWith("/")) {
         return url;
      } else {
         file = trailingSlash ? file + "/" : file.substring(0, file.length() - 1);
         return new URL(url.getProtocol(), url.getHost(), file);
      }
   }

   private URL buildLocation(String property, URL defaultLocation, String userDefaultAppendage) {
      URL result = null;
      String location = System.getProperty(property);
      System.getProperties().remove(property);

      try {
         if (location == null) {
            result = defaultLocation;
            return result;
         }

         if (!location.equalsIgnoreCase("@none")) {
            if (location.equalsIgnoreCase("@noDefault")) {
               result = buildURL(location, true);
            } else {
               String base;
               if (location.startsWith("@user.home")) {
                  base = this.substituteVar(location, "@user.home", "user.home");
                  location = (new File(base, userDefaultAppendage)).getAbsolutePath();
               } else if (location.startsWith("@user.dir")) {
                  base = this.substituteVar(location, "@user.dir", "user.dir");
                  location = (new File(base, userDefaultAppendage)).getAbsolutePath();
               }

               result = buildURL(location, true);
            }

            return result;
         }
      } finally {
         if (result != null) {
            System.getProperties().put(property, result.toExternalForm());
         }

      }

      return null;
   }

   private String substituteVar(String source, String var, String prop) {
      String value = System.getProperty(prop, "");
      return value + source.substring(var.length());
   }

   private String computeDefaultConfigurationLocation() {
      URL install = this.getInstallLocation();
      if (install.getProtocol().equals("file")) {
         File installDir = new File(install.getFile());
         if (canWrite(installDir)) {
            return installDir.getAbsolutePath() + File.separator + "configuration/";
         }
      }

      return this.computeDefaultUserAreaLocation("configuration/");
   }

   private static boolean canWrite(File installDir) {
      if (!installDir.canWrite()) {
         return false;
      } else if (!installDir.isDirectory()) {
         return false;
      } else {
         File fileTest = null;

         try {
            fileTest = File.createTempFile("writtableArea", ".dll", installDir);
            return true;
         } catch (IOException var5) {
         } finally {
            if (fileTest != null) {
               fileTest.delete();
            }

         }

         return false;
      }
   }

   private String computeDefaultUserAreaLocation(String pathAppendage) {
      URL installURL = this.getInstallLocation();
      if (installURL == null) {
         return null;
      } else {
         File installDir = new File(installURL.getFile());

         int hashCode;
         try {
            hashCode = installDir.getCanonicalPath().hashCode();
         } catch (IOException var11) {
            hashCode = installDir.getAbsolutePath().hashCode();
         }

         if (hashCode < 0) {
            hashCode = -hashCode;
         }

         String installDirHash = String.valueOf(hashCode);
         String appName = ".eclipse";
         File eclipseProduct = new File(installDir, ".eclipseproduct");
         if (eclipseProduct.exists()) {
            Properties props = new Properties();

            try {
               props.load(new FileInputStream(eclipseProduct));
               String appId = props.getProperty("id");
               if (appId == null || appId.trim().length() == 0) {
                  appId = "eclipse";
               }

               String appVersion = props.getProperty("version");
               if (appVersion == null || appVersion.trim().length() == 0) {
                  appVersion = "";
               }

               appName = appName + File.separator + appId + "_" + appVersion + "_" + installDirHash;
            } catch (IOException var12) {
               appName = appName + File.separator + installDirHash;
            }
         } else {
            appName = appName + File.separator + installDirHash;
         }

         String userHome = System.getProperty("user.home");
         return (new File(userHome, appName + "/" + pathAppendage)).getAbsolutePath();
      }
   }

   public static void main(String argString) {
      Vector list = new Vector(5);
      StringTokenizer tokens = new StringTokenizer(argString, " ");

      while(tokens.hasMoreElements()) {
         list.addElement(tokens.nextElement());
      }

      main((String[])list.toArray(new String[list.size()]));
   }

   public static void main(String[] args) {
      int result = false;

      try {
         try {
            int result = (new Main()).run(args);
         } catch (Throwable var6) {
            Throwable t = var6;
            t.printStackTrace();
         }

      } finally {
         ;
      }
   }

   public int run(String[] args) {
      int result = false;

      int result;
      try {
         this.basicRun(args);
         String exitCode = System.getProperty("eclipse.exitcode");

         try {
            result = exitCode == null ? 0 : Integer.parseInt(exitCode);
         } catch (NumberFormatException var9) {
            result = 17;
         }
      } catch (Throwable var10) {
         Throwable e = var10;
         if (!"13".equals(System.getProperty("eclipse.exitcode"))) {
            this.log("Exception launching the Eclipse Platform:");
            this.log(e);
            String message = "An error has occurred";
            if (this.logFile == null) {
               message = message + " and could not be logged: \n" + e.getMessage();
            } else {
               message = message + ".  See the log file\n" + this.logFile.getAbsolutePath();
            }

            System.getProperties().put("eclipse.exitdata", message);
         }

         result = 13;
      } finally {
         this.takeDownSplash();
         if (this.bridge != null) {
            this.bridge.uninitialize();
         }

      }

      System.getProperties().put("eclipse.exitcode", Integer.toString(result));
      this.setExitData();
      return result;
   }

   private void setExitData() {
      String data = System.getProperty("eclipse.exitdata");
      if (data != null && this.bridge != null) {
         this.bridge.setExitData(this.exitData, data);
      }
   }

   protected String[] processCommandLine(String[] args) {
      if (args.length == 0) {
         return args;
      } else {
         int[] configArgs = new int[args.length];
         configArgs[0] = -1;
         int configArgIndex = 0;

         int j;
         for(int i = 0; i < args.length; ++i) {
            boolean found = false;
            if (args[i].equalsIgnoreCase("-debug")) {
               this.debug = true;
            } else {
               if (args[i].equalsIgnoreCase("-nosplash")) {
                  this.splashDown = true;
                  found = true;
               }

               if (args[i].equalsIgnoreCase("-noExit")) {
                  System.getProperties().put("osgi.noShutdown", "true");
                  found = true;
               }

               if (args[i].equalsIgnoreCase("-initialize")) {
                  this.initialize = true;
               } else if (!args[i].equalsIgnoreCase("-dev") || i + 1 != args.length && (i + 1 >= args.length || !args[i + 1].startsWith("-"))) {
                  if (args[i].equalsIgnoreCase("-showsplash")) {
                     this.showSplash = true;
                     found = true;
                     if (i + 1 < args.length && !args[i + 1].startsWith("-")) {
                        configArgs[configArgIndex++] = i++;
                        this.splashLocation = args[i];
                     }
                  }

                  if (found) {
                     configArgs[configArgIndex++] = i;
                  } else if (args[i].equalsIgnoreCase("-vmargs")) {
                     args[i] = null;
                     ++i;
                     this.vmargs = new String[args.length - i];

                     for(j = 0; i < args.length; ++i) {
                        this.vmargs[j++] = args[i];
                        args[i] = null;
                     }
                  } else if (i != args.length - 1 && !args[i + 1].startsWith("-")) {
                     ++i;
                     String arg = args[i];
                     if (args[i - 1].equalsIgnoreCase("-dev")) {
                        this.inDevelopmentMode = true;
                        this.devClassPathProps = this.processDevArg(arg);
                        if (this.devClassPathProps != null) {
                           this.devClassPath = this.devClassPathProps.getProperty("org.eclipse.osgi");
                           if (this.devClassPath == null) {
                              this.devClassPath = this.devClassPathProps.getProperty("*");
                           }
                        }
                     } else {
                        if (args[i - 1].equalsIgnoreCase("-framework")) {
                           this.framework = arg;
                           found = true;
                        }

                        if (args[i - 1].equalsIgnoreCase("-os")) {
                           this.os = arg;
                           System.getProperties().put("os.type", this.os);
                        } else if (args[i - 1].equalsIgnoreCase("-ws")) {
                           this.ws = arg;
                        } else if (args[i - 1].equalsIgnoreCase("-arch")) {
                           this.arch = arg;
                        } else {
                           if (args[i - 1].equalsIgnoreCase("-install")) {
                              System.getProperties().put("osgi.install.area", arg);
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-configuration")) {
                              System.getProperties().put("osgi.configuration.area", arg);
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-exitdata")) {
                              this.exitData = arg;
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-name")) {
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-startup")) {
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-launcher")) {
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("--launcher.library")) {
                              this.library = arg;
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-endsplash")) {
                              this.endSplash = arg;
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-vm")) {
                              this.vm = arg;
                              found = true;
                           }

                           if (args[i - 1].equalsIgnoreCase("-nl")) {
                              System.getProperties().put("osgi.nl", arg);
                              found = true;
                           }

                           if (found) {
                              configArgs[configArgIndex++] = i - 1;
                              configArgs[configArgIndex++] = i;
                           }
                        }
                     }
                  }
               } else {
                  this.inDevelopmentMode = true;
               }
            }
         }

         String[] passThruArgs = new String[args.length - configArgIndex - (this.vmargs == null ? 0 : this.vmargs.length + 1)];
         configArgIndex = 0;
         int j = 0;

         for(j = 0; j < args.length; ++j) {
            if (j == configArgs[configArgIndex]) {
               ++configArgIndex;
            } else if (args[j] != null) {
               passThruArgs[j++] = args[j];
            }
         }

         return passThruArgs;
      }
   }

   private Properties processDevArg(String arg) {
      if (arg == null) {
         return null;
      } else {
         try {
            URL location = new URL(arg);
            return this.load(location, (String)null);
         } catch (MalformedURLException var3) {
            Properties result = new Properties();
            result.put("*", arg);
            return result;
         } catch (IOException var4) {
            return null;
         }
      }
   }

   private URL getConfigurationLocation() {
      if (this.configurationLocation != null) {
         return this.configurationLocation;
      } else {
         this.configurationLocation = this.buildLocation("osgi.configuration.area", (URL)null, "");
         if (this.configurationLocation == null) {
            this.configurationLocation = this.buildLocation("osgi.configuration.area.default", (URL)null, "");
            if (this.configurationLocation == null) {
               this.configurationLocation = buildURL(this.computeDefaultConfigurationLocation(), true);
            }
         }

         if (this.configurationLocation != null) {
            System.getProperties().put("osgi.configuration.area", this.configurationLocation.toExternalForm());
         }

         if (this.debug) {
            System.out.println("Configuration location:\n    " + this.configurationLocation);
         }

         return this.configurationLocation;
      }
   }

   private void processConfiguration() {
      URL baseConfigurationLocation = null;
      Properties baseConfiguration = null;
      String location;
      if (System.getProperty("osgi.configuration.area") == null) {
         String baseLocation = System.getProperty("osgi.baseConfiguration.area");
         if (baseLocation != null) {
            baseConfigurationLocation = buildURL(baseLocation, true);
         }

         if (baseConfigurationLocation == null) {
            try {
               baseConfigurationLocation = new URL(this.getInstallLocation(), "configuration/");
            } catch (MalformedURLException var7) {
            }
         }

         baseConfiguration = this.loadConfiguration(baseConfigurationLocation);
         if (baseConfiguration != null) {
            location = baseConfiguration.getProperty("osgi.configuration.area");
            if (location != null) {
               System.getProperties().put("osgi.configuration.area", location);
            }

            location = baseConfiguration.getProperty("osgi.install.area");
            if (location != null && System.getProperty("osgi.install.area") == null) {
               System.getProperties().put("osgi.install.area", location);
            }
         }
      }

      Properties configuration = baseConfiguration;
      if (configuration == null || !this.getConfigurationLocation().equals(baseConfigurationLocation)) {
         configuration = this.loadConfiguration(this.getConfigurationLocation());
      }

      this.mergeProperties(System.getProperties(), configuration);
      if ("false".equalsIgnoreCase(System.getProperty("osgi.configuration.cascaded"))) {
         System.getProperties().remove("osgi.sharedConfiguration.area");
      } else {
         this.ensureAbsolute("osgi.sharedConfiguration.area");
         URL sharedConfigURL = this.buildLocation("osgi.sharedConfiguration.area", (URL)null, "");
         if (sharedConfigURL == null) {
            try {
               sharedConfigURL = new URL(this.getInstallLocation(), "configuration/");
            } catch (MalformedURLException var6) {
            }
         }

         if (sharedConfigURL != null) {
            if (sharedConfigURL.equals(this.getConfigurationLocation())) {
               System.getProperties().remove("osgi.sharedConfiguration.area");
            } else {
               configuration = baseConfiguration;
               if (!sharedConfigURL.equals(baseConfigurationLocation)) {
                  configuration = this.loadConfiguration(sharedConfigURL);
               }

               this.mergeProperties(System.getProperties(), configuration);
               System.getProperties().put("osgi.sharedConfiguration.area", sharedConfigURL.toExternalForm());
               if (this.debug) {
                  System.out.println("Shared configuration location:\n    " + sharedConfigURL.toExternalForm());
               }
            }
         }
      }

      location = System.getProperty("osgi.framework", (String)null);
      if (location != null) {
         URL url = buildURL(location, true);
         System.getProperties().put("osgi.framework", url.toExternalForm());
         this.bootLocation = this.resolve(location);
      }

   }

   private void ensureAbsolute(String locationProperty) {
      String propertyValue = System.getProperty(locationProperty);
      if (propertyValue != null) {
         URL locationURL = null;

         try {
            locationURL = new URL(propertyValue);
         } catch (MalformedURLException var8) {
            return;
         }

         String locationPath = locationURL.getPath();
         if (!locationPath.startsWith("/")) {
            URL installURL = this.getInstallLocation();
            if (locationURL.getProtocol().equals(installURL.getProtocol())) {
               try {
                  URL absoluteURL = new URL(installURL, locationPath);
                  System.getProperties().put(locationProperty, absoluteURL.toExternalForm());
               } catch (MalformedURLException var7) {
               }

            }
         }
      }
   }

   private URL getInstallLocation() {
      if (this.installLocation != null) {
         return this.installLocation;
      } else {
         String installArea = System.getProperty("osgi.install.area");
         if (installArea != null) {
            this.installLocation = buildURL(installArea, true);
            if (this.installLocation == null) {
               throw new IllegalStateException("Install location is invalid: " + installArea);
            } else {
               System.getProperties().put("osgi.install.area", this.installLocation.toExternalForm());
               if (this.debug) {
                  System.out.println("Install location:\n    " + this.installLocation);
               }

               return this.installLocation;
            }
         } else {
            ProtectionDomain domain = Main.class.getProtectionDomain();
            CodeSource source = null;
            URL result = null;
            if (domain != null) {
               source = domain.getCodeSource();
            }

            if (source == null || domain == null) {
               if (this.debug) {
                  System.out.println("CodeSource location is null. Defaulting the install location to file:startup.jar");
               }

               try {
                  result = new URL("file:startup.jar");
               } catch (MalformedURLException var10) {
               }
            }

            if (source != null) {
               result = source.getLocation();
            }

            String path = this.decode(result.getFile());
            File file = new File(path);
            path = file.toString().replace('\\', '/');
            if (File.separatorChar == '\\' && Character.isUpperCase(path.charAt(0))) {
               char[] chars = path.toCharArray();
               chars[0] = Character.toLowerCase(chars[0]);
               path = new String(chars);
            }

            System.setProperty("com.jiuqi.dna.launcher.path", file.getAbsolutePath());
            if (path.toLowerCase().endsWith(".jar")) {
               path = path.substring(0, path.lastIndexOf("/") + 1);
            }

            if (path.toLowerCase().endsWith("/bundles/")) {
               path = path.substring(0, path.length() - "/bundles/".length());
            } else if (path.toLowerCase().endsWith("/thr/")) {
               path = path.substring(0, path.length() - "/thr/".length());
            }

            if (path.contains("#") || path.contains("%") || path.contains("!")) {
               System.err.println("Install path: " + path);
               System.err.println("ERROR: Install path contains illegal characters like \"#%!\".");
               System.exit(1);
            }

            try {
               try {
                  path = (new File(path)).toURL().getFile();
               } catch (MalformedURLException var8) {
               }

               this.installLocation = new URL(result.getProtocol(), result.getHost(), result.getPort(), path);
               System.getProperties().put("osgi.install.area", this.installLocation.toExternalForm());
            } catch (MalformedURLException var9) {
            }

            if (this.debug) {
               System.out.println("Install location:\n    " + this.installLocation);
            }

            return this.installLocation;
         }
      }
   }

   private Properties loadConfiguration(URL url) {
      Properties result = null;

      try {
         url = new URL(url, "config.ini");
      } catch (MalformedURLException var3) {
         return null;
      }

      try {
         if (this.debug) {
            System.out.print("Configuration file:\n    " + url.toString());
         }

         result = this.loadProperties(url);
         if (this.debug) {
            System.out.println(" loaded");
         }
      } catch (IOException var4) {
         if (this.debug) {
            System.out.println(" not found or not read");
         }
      }

      return result;
   }

   private Properties loadProperties(URL url) throws IOException {
      if (url == null) {
         return null;
      } else {
         Properties result = null;
         IOException originalException = null;

         try {
            result = this.load(url, (String)null);
         } catch (IOException var7) {
            IOException e1 = var7;
            originalException = e1;

            try {
               result = this.load(url, ".tmp");
            } catch (IOException var6) {
               try {
                  result = this.load(url, ".bak");
               } catch (IOException var5) {
                  throw originalException;
               }
            }
         }

         return result;
      }
   }

   private Properties load(URL url, String suffix) throws IOException {
      if (suffix != null && !suffix.equals("")) {
         url = new URL(url.getProtocol(), url.getHost(), url.getPort(), url.getFile() + suffix);
      }

      Properties props = new Properties();
      InputStream is = null;

      try {
         is = url.openStream();
         props.load(is);
      } finally {
         if (is != null) {
            try {
               is.close();
            } catch (IOException var9) {
            }
         }

      }

      return props;
   }

   private void handleSplash(URL[] defaultPath) {
      if (!this.initialize && !this.splashDown && this.bridge != null) {
         if (this.showSplash || this.endSplash != null) {
            try {
               Runtime.getRuntime().addShutdownHook(this.splashHandler);
            } catch (Throwable var4) {
            }
         }

         if (this.endSplash != null) {
            this.showSplash = false;
         } else if (this.showSplash) {
            this.splashLocation = this.getSplashLocation(defaultPath);
            if (this.debug) {
               System.out.println("Splash location:\n    " + this.splashLocation);
            }

            if (this.splashLocation != null) {
               this.bridge.showSplash(this.splashLocation);
               long handle = this.bridge.getSplashHandle();
               if (handle != 0L && handle != -1L) {
                  System.getProperties().put("org.eclipse.equinox.launcher.splash.handle", String.valueOf(handle));
                  System.getProperties().put("org.eclipse.equinox.launcher.splash.location", this.splashLocation);
                  this.bridge.updateSplash();
               }

            }
         }
      } else {
         this.showSplash = false;
         this.endSplash = null;
      }
   }

   protected void takeDownSplash() {
      if (!this.splashDown && this.bridge != null) {
         this.splashDown = this.bridge.takeDownSplash();
         System.getProperties().remove("org.eclipse.equinox.launcher.splash.handle");

         try {
            Runtime.getRuntime().removeShutdownHook(this.splashHandler);
         } catch (Throwable var1) {
         }

      }
   }

   private String getSplashLocation(URL[] bootPath) {
      if (this.splashLocation != null && !Character.isDigit(this.splashLocation.charAt(0)) && (new File(this.splashLocation)).exists()) {
         System.getProperties().put("osgi.splashLocation", this.splashLocation);
         return this.splashLocation;
      } else {
         String result = System.getProperty("osgi.splashLocation");
         if (result != null) {
            return result;
         } else {
            String splashPath = System.getProperty("osgi.splashPath");
            if (splashPath != null) {
               String[] entries = this.getArrayFromList(splashPath);
               ArrayList path = new ArrayList(entries.length);

               for(int i = 0; i < entries.length; ++i) {
                  String entry = this.resolve(entries[i]);
                  if (entry != null && !entry.startsWith("file:")) {
                     this.log("Invalid splash path entry: " + entries[i]);
                  } else {
                     File entryFile = new File(entry.substring(5).replace('/', File.separatorChar));
                     entry = this.searchFor(entryFile.getName(), entryFile.getParent());
                     if (entry != null) {
                        path.add(entry);
                     }
                  }
               }

               result = this.searchForSplash((String[])path.toArray(new String[path.size()]));
               if (result != null) {
                  System.getProperties().put("osgi.splashLocation", result);
                  return result;
               }
            }

            result = this.searchSplash(bootPath, "bundles");
            if (result == null) {
               result = this.searchSplash(bootPath, "thr");
            }

            return result;
         }
      }
   }

   private String searchSplash(URL[] bootPath, String dir) {
      String result = null;
      String temp = bootPath[0].getFile();
      temp = temp.replace('/', File.separatorChar);
      int ix = temp.lastIndexOf(dir + File.separator);
      if (ix != -1) {
         int pix = temp.indexOf(File.separator, ix + 8);
         if (pix != -1) {
            temp = temp.substring(0, pix);
            result = this.searchForSplash(new String[]{temp});
            if (result != null) {
               System.getProperties().put("osgi.splashLocation", result);
            }
         }
      }

      return result;
   }

   private String searchForSplash(String[] searchPath) {
      if (searchPath == null) {
         return null;
      } else {
         String locale = (String)System.getProperties().get("osgi.nl");
         if (locale == null) {
            locale = Locale.getDefault().toString();
         }

         String[] nlVariants = buildNLVariants(locale);

         for(int i = 0; i < nlVariants.length; ++i) {
            for(int j = 0; j < searchPath.length; ++j) {
               String result;
               if (this.isJAR(searchPath[j])) {
                  result = this.extractSplashFromJAR(searchPath[j], nlVariants[i]);
                  if (result != null) {
                     return result;
                  }
               } else {
                  result = searchPath[j];
                  if (!result.endsWith(File.separator)) {
                     result = result + File.separator;
                  }

                  result = result + nlVariants[i];
                  File result = new File(result);
                  if (result.exists()) {
                     return result.getAbsolutePath();
                  }
               }
            }
         }

         return null;
      }
   }

   private static void transferStreams(InputStream source, OutputStream destination) {
      byte[] buffer = new byte[8096];

      while(true) {
         try {
            int bytesRead = true;

            int bytesRead;
            try {
               bytesRead = source.read(buffer);
            } catch (IOException var105) {
               return;
            }

            if (bytesRead == -1) {
               return;
            }

            try {
               destination.write(buffer, 0, bytesRead);
               continue;
            } catch (IOException var104) {
            }
         } finally {
            try {
               source.close();
            } catch (IOException var102) {
            } finally {
               try {
                  destination.close();
               } catch (IOException var101) {
               }

            }

         }

         return;
      }
   }

   private String extractSplashFromJAR(String jarPath, String splashPath) {
      String configLocation = System.getProperty("osgi.configuration.area");
      if (configLocation == null) {
         this.log("Configuration area not set yet. Unable to extract splash from JAR'd plug-in: " + jarPath);
         return null;
      } else {
         URL configURL = buildURL(configLocation, false);
         if (configURL == null) {
            return null;
         } else {
            File splash = new File(configURL.getPath(), "com.jiuqi.dna.launcher");
            File jarFile = new File(jarPath);
            String cache = jarFile.getName();
            if (cache.endsWith(".jar")) {
               cache = cache.substring(0, cache.length() - 4);
            }

            splash = new File(splash, cache);
            splash = new File(splash, splashPath);
            if (splash.exists()) {
               boolean clean = false;

               for(int i = 0; i < this.commands.length; ++i) {
                  if ("-clean".equalsIgnoreCase(this.commands[i])) {
                     clean = true;
                     splash.delete();
                     break;
                  }
               }

               if (!clean) {
                  return splash.getAbsolutePath();
               }
            }

            ZipFile file;
            try {
               file = new ZipFile(jarPath);
            } catch (IOException var15) {
               this.log("Exception looking for splash  in JAR file: " + jarPath);
               this.log(var15);
               return null;
            }

            ZipEntry entry = file.getEntry(splashPath.replace(File.separatorChar, '/'));
            if (entry == null) {
               return null;
            } else {
               InputStream input = null;

               try {
                  input = file.getInputStream(entry);
               } catch (IOException var14) {
                  IOException e = var14;
                  this.log("Exception opening splash: " + entry.getName() + " in JAR file: " + jarPath);
                  this.log(e);
                  return null;
               }

               (new File(splash.getParent())).mkdirs();

               BufferedOutputStream output;
               try {
                  output = new BufferedOutputStream(new FileOutputStream(splash));
               } catch (FileNotFoundException var13) {
                  try {
                     input.close();
                  } catch (IOException var12) {
                  }

                  return null;
               }

               transferStreams(input, output);
               return splash.exists() ? splash.getAbsolutePath() : null;
            }
         }
      }
   }

   private boolean isJAR(String path) {
      if (path.endsWith(File.separator)) {
         return false;
      } else {
         int index = path.lastIndexOf(46);
         if (index == -1) {
            return false;
         } else {
            ++index;
            return index >= path.length() ? false : "JAR".equalsIgnoreCase(path.substring(index));
         }
      }
   }

   private static String[] buildNLVariants(String locale) {
      String nl = locale;
      ArrayList result = new ArrayList(4);

      while(true) {
         result.add("nl" + File.separatorChar + nl.replace('_', File.separatorChar) + File.separatorChar + "splash.bmp");
         int lastSeparator = nl.lastIndexOf(95);
         if (lastSeparator == -1) {
            result.add("splash.bmp");
            return (String[])result.toArray(new String[result.size()]);
         }

         nl = nl.substring(0, lastSeparator);
      }
   }

   private String resolve(String urlString) {
      if (urlString.startsWith("reference:")) {
         urlString = urlString.substring(10);
         System.getProperties().put("osgi.framework", urlString);
      }

      if (urlString.startsWith("platform:/base/")) {
         String path = urlString.substring("platform:/base/".length());
         return this.getInstallLocation() + path;
      } else {
         return urlString;
      }
   }

   protected synchronized void log(Object obj) {
      if (obj != null) {
         try {
            this.openLogFile();

            try {
               if (this.newSession) {
                  this.log.write("!SESSION");
                  this.log.write(32);
                  String timestamp = (new Date()).toString();
                  this.log.write(timestamp);
                  this.log.write(32);

                  for(int i = "!SESSION".length() + timestamp.length(); i < 78; ++i) {
                     this.log.write(45);
                  }

                  this.log.newLine();
                  this.newSession = false;
               }

               this.write(obj);
            } finally {
               if (this.logFile == null) {
                  if (this.log != null) {
                     this.log.flush();
                  }
               } else {
                  this.closeLogFile();
               }

            }
         } catch (Exception var16) {
            Exception e = var16;
            System.err.println("An exception occurred while writing to the platform log:");
            e.printStackTrace(System.err);
            System.err.println("Logging to the console instead.");

            try {
               this.log = this.logForStream(System.err);
               this.write(obj);
               this.log.flush();
            } catch (Exception var14) {
               Exception e2 = var14;
               System.err.println("An exception occurred while logging to the console:");
               e2.printStackTrace(System.err);
            }
         } finally {
            this.log = null;
         }

      }
   }

   private void write(Object obj) throws IOException {
      if (obj != null) {
         if (obj instanceof Throwable) {
            this.log.write("!STACK");
            this.log.newLine();
            ((Throwable)obj).printStackTrace(new PrintWriter(this.log));
         } else {
            this.log.write("!ENTRY");
            this.log.write(32);
            this.log.write("com.jiuqi.dna.launcher");
            this.log.write(32);
            this.log.write(String.valueOf(4));
            this.log.write(32);
            this.log.write(String.valueOf(0));
            this.log.write(32);
            this.log.write(this.getDate(new Date()));
            this.log.newLine();
            this.log.write("!MESSAGE");
            this.log.write(32);
            this.log.write(String.valueOf(obj));
         }

         this.log.newLine();
      }
   }

   protected String getDate(Date date) {
      Calendar c = Calendar.getInstance();
      c.setTime(date);
      StringBuffer sb = new StringBuffer();
      this.appendPaddedInt(c.get(1), 4, sb).append('-');
      this.appendPaddedInt(c.get(2) + 1, 2, sb).append('-');
      this.appendPaddedInt(c.get(5), 2, sb).append(' ');
      this.appendPaddedInt(c.get(11), 2, sb).append(':');
      this.appendPaddedInt(c.get(12), 2, sb).append(':');
      this.appendPaddedInt(c.get(13), 2, sb).append('.');
      this.appendPaddedInt(c.get(14), 3, sb);
      return sb.toString();
   }

   private StringBuffer appendPaddedInt(int value, int pad, StringBuffer buffer) {
      --pad;
      if (pad == 0) {
         return buffer.append(Integer.toString(value));
      } else {
         int padding = (int)Math.pow(10.0, (double)pad);
         if (value >= padding) {
            return buffer.append(Integer.toString(value));
         } else {
            while(padding > value && padding > 1) {
               buffer.append('0');
               padding /= 10;
            }

            buffer.append(value);
            return buffer;
         }
      }
   }

   private void computeLogFileLocation() {
      String logFileProp = System.getProperty("osgi.logfile");
      if (logFileProp == null) {
         URL base = buildURL(System.getProperty("osgi.configuration.area"), false);
         if (base != null) {
            this.logFile = new File(base.getPath(), Long.toString(System.currentTimeMillis()) + ".log");
            (new File(this.logFile.getParent())).mkdirs();
            System.getProperties().put("osgi.logfile", this.logFile.getAbsolutePath());
         }
      } else {
         if (this.logFile == null || !logFileProp.equals(this.logFile.getAbsolutePath())) {
            this.logFile = new File(logFileProp);
            (new File(this.logFile.getParent())).mkdirs();
         }

      }
   }

   private int hexToByte(byte b) {
      switch (b) {
         case 48:
            return 0;
         case 49:
            return 1;
         case 50:
            return 2;
         case 51:
            return 3;
         case 52:
            return 4;
         case 53:
            return 5;
         case 54:
            return 6;
         case 55:
            return 7;
         case 56:
            return 8;
         case 57:
            return 9;
         case 58:
         case 59:
         case 60:
         case 61:
         case 62:
         case 63:
         case 64:
         case 71:
         case 72:
         case 73:
         case 74:
         case 75:
         case 76:
         case 77:
         case 78:
         case 79:
         case 80:
         case 81:
         case 82:
         case 83:
         case 84:
         case 85:
         case 86:
         case 87:
         case 88:
         case 89:
         case 90:
         case 91:
         case 92:
         case 93:
         case 94:
         case 95:
         case 96:
         default:
            throw new IllegalArgumentException("Switch error decoding URL");
         case 65:
         case 97:
            return 10;
         case 66:
         case 98:
            return 11;
         case 67:
         case 99:
            return 12;
         case 68:
         case 100:
            return 13;
         case 69:
         case 101:
            return 14;
         case 70:
         case 102:
            return 15;
      }
   }

   private void openLogFile() throws IOException {
      this.computeLogFileLocation();

      try {
         this.log = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(this.logFile.getAbsolutePath(), true), "UTF-8"));
      } catch (IOException var2) {
         IOException e = var2;
         this.logFile = null;
         throw e;
      }
   }

   private BufferedWriter logForStream(OutputStream output) {
      try {
         return new BufferedWriter(new OutputStreamWriter(output, "UTF-8"));
      } catch (UnsupportedEncodingException var2) {
         return new BufferedWriter(new OutputStreamWriter(output));
      }
   }

   private void closeLogFile() throws IOException {
      try {
         if (this.log != null) {
            this.log.flush();
            this.log.close();
         }
      } finally {
         this.log = null;
      }

   }

   private void mergeProperties(Properties destination, Properties source) {
      if (destination != null && source != null) {
         Enumeration e = source.keys();

         while(e.hasMoreElements()) {
            String key = (String)e.nextElement();
            String destinationClasspath;
            if (key.equals("osgi.frameworkClassPath")) {
               destinationClasspath = destination.getProperty("osgi.frameworkClassPath");
               String sourceClasspath = source.getProperty("osgi.frameworkClassPath");
               if (destinationClasspath == null) {
                  destinationClasspath = sourceClasspath;
               } else {
                  destinationClasspath = destinationClasspath + sourceClasspath;
               }

               destination.put("osgi.frameworkClassPath", destinationClasspath);
            } else {
               destinationClasspath = source.getProperty(key);
               if (destination.getProperty(key) == null) {
                  destination.put(key, destinationClasspath);
               }
            }
         }

      }
   }

   private void setupVMProperties() {
      if (this.vm != null) {
         System.getProperties().put("eclipse.vm", this.vm);
      }

      this.setMultiValueProperty("eclipse.vmargs", this.vmargs);
      this.setMultiValueProperty("eclipse.commands", this.commands);
   }

   private void setMultiValueProperty(String property, String[] value) {
      if (value != null) {
         StringBuffer result = new StringBuffer(300);

         for(int i = 0; i < value.length; ++i) {
            if (value[i] != null) {
               result.append(value[i]);
               result.append('\n');
            }
         }

         System.getProperties().put(property, result.toString());
      }

   }

   private class EclipsePolicy extends Policy {
      private final Policy policy;
      private final URL[] urls;
      private final PermissionCollection allPermissions;
      Permission allPermission = new AllPermission();

      EclipsePolicy(Policy policy, URL[] urls) {
         this.policy = policy;
         this.urls = urls;
         this.allPermissions = new PermissionCollection() {
            private static final long serialVersionUID = 3258131349494708277L;

            public void add(Permission permission) {
            }

            public boolean implies(Permission permission) {
               return true;
            }

            public Enumeration elements() {
               return new Enumeration() {
                  int cur = 0;

                  public boolean hasMoreElements() {
                     return this.cur < 1;
                  }

                  public Object nextElement() {
                     if (this.cur == 0) {
                        this.cur = 1;
                        return EclipsePolicy.this.allPermission;
                     } else {
                        throw new NoSuchElementException();
                     }
                  }
               };
            }
         };
      }

      public PermissionCollection getPermissions(CodeSource codesource) {
         if (this.contains(codesource.getLocation())) {
            return this.allPermissions;
         } else {
            return this.policy == null ? this.allPermissions : this.policy.getPermissions(codesource);
         }
      }

      public PermissionCollection getPermissions(ProtectionDomain domain) {
         if (this.contains(domain.getCodeSource().getLocation())) {
            return this.allPermissions;
         } else {
            return this.policy == null ? this.allPermissions : this.policy.getPermissions(domain);
         }
      }

      public boolean implies(ProtectionDomain domain, Permission permission) {
         if (this.contains(domain.getCodeSource().getLocation())) {
            return true;
         } else {
            return this.policy == null ? true : this.policy.implies(domain, permission);
         }
      }

      public void refresh() {
         if (this.policy != null) {
            this.policy.refresh();
         }

      }

      private boolean contains(URL url) {
         for(int i = 0; i < this.urls.length; ++i) {
            if (this.urls[i] == url) {
               return true;
            }
         }

         return false;
      }
   }

   static class Identifier {
      private static final String DELIM = ". _-";
      private int major;
      private int minor;
      private int service;

      Identifier(int major, int minor, int service) {
         this.major = major;
         this.minor = minor;
         this.service = service;
      }

      Identifier(String versionString) {
         StringTokenizer tokenizer = new StringTokenizer(versionString, ". _-");
         if (tokenizer.hasMoreTokens()) {
            this.major = Integer.parseInt(tokenizer.nextToken());
         }

         if (tokenizer.hasMoreTokens()) {
            this.minor = Integer.parseInt(tokenizer.nextToken());
         }

         try {
            if (tokenizer.hasMoreTokens()) {
               this.service = Integer.parseInt(tokenizer.nextToken());
            }
         } catch (NumberFormatException var3) {
         }

      }

      boolean isGreaterEqualTo(Identifier minimum) {
         if (this.major < minimum.major) {
            return false;
         } else if (this.major > minimum.major) {
            return true;
         } else if (this.minor < minimum.minor) {
            return false;
         } else if (this.minor > minimum.minor) {
            return true;
         } else {
            return this.service >= minimum.service;
         }
      }
   }

   public final class SplashHandler extends Thread {
      public void run() {
         Main.this.takeDownSplash();
      }

      public void updateSplash() {
         if (Main.this.bridge != null) {
            Main.this.bridge.updateSplash();
         }

      }
   }

   private class StartupClassLoader extends URLClassLoader {
      public StartupClassLoader(URL[] urls) {
         super(urls);
      }

      public StartupClassLoader(URL[] urls, ClassLoader parent) {
         super(urls, parent);
      }

      public StartupClassLoader(URL[] urls, ClassLoader parent, URLStreamHandlerFactory factory) {
         super(urls, parent, factory);
      }

      protected String findLibrary(String name) {
         if (Main.this.extensionPaths == null) {
            return super.findLibrary(name);
         } else {
            String libName = System.mapLibraryName(name);

            for(int i = 0; i < Main.this.extensionPaths.length; ++i) {
               File libFile = new File(Main.this.extensionPaths[i], libName);
               if (libFile.isFile()) {
                  return libFile.getAbsolutePath();
               }
            }

            return super.findLibrary(name);
         }
      }
   }
}
